//
//  MSGoogleChartUtility.h
//  Galileo-GM
//
//  Created by Drew Breunig on 11/3/11.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>

@class MSGoogleChartDataSet;

typedef enum {
  // Bar Chart
  MSGoogleChartTypeBarHorizontalStacked = 0,
  MSGoogleChartTypeBarVerticalStacked,
  MSGoogleChartTypeBarVerticalOverlap,
  MSGoogleChartTypeBarHorizontalGrouped,
  MSGoogleChartTypeBarVerticalGrouped,
  // Line Charts
  MSGoogleChartTypeLineChart,
  MSGoogleChartTypeLineChartSparkLine,
  MSGoogleChartTypeLineChartXY,
  // Pie Charts
  MSGoogleChartTypePie,
  MSGoogleChartTypePie3D,
  MSGoogleChartTypePieConcentric,
  // Visualization API
  MSGoogleChartTypeThematicMap
} MSGoogleChartType;

typedef enum {
  MSGoogleChartLegendPositionLeft = 0,
  MSGoogleChartLegendPositionRight,
  MSGoogleChartLegendPositionTop,
  MSGoogleChartLegendPositionBottom  
} MSGoogleChartLegendPosition;

typedef enum {
  MSGoogleChartMapGeographicAreaAfrica = 0,
  MSGoogleChartMapGeographicAreaAsia,
  MSGoogleChartMapGeographicAreaMiddleEast,
  MSGoogleChartMapGeographicAreaSouthAmerica,
  MSGoogleChartMapGeographicAreaUSA,
  MSGoogleChartMapGeographicAreaWorld
} MSGoogleChartMapGeographicArea;

@interface MSGoogleChartUtility : NSObject

@property (strong, nonatomic) NSString *baseUrl;

#pragma mark - Global Properties

@property (nonatomic) MSGoogleChartType chartType;

#pragma mark Chart Title
@property (strong, nonatomic) NSString *chartTitle;
@property (nonatomic) float chartTitleFontSize;
@property (strong, nonatomic) UIColor *chartTitleColor;

#pragma mark Chart Size
@property (nonatomic) int chartWidth;
@property (nonatomic) int chartHeight;
@property (nonatomic) int leftMargin;
@property (nonatomic) int rightMargin;
@property (nonatomic) int topMargin;
@property (nonatomic) int bottomMargin;

#pragma mark Fill Colors
@property (strong, nonatomic) UIColor *backGroundColor;
@property (strong, nonatomic) UIColor *chartAreaColor;

#pragma mark Data Sets and Axes
@property (strong, nonatomic) NSMutableArray *dataSets;
@property (strong, nonatomic) NSMutableArray *axes;

#pragma mark Legend
@property (nonatomic) MSGoogleChartLegendPosition legendPosition;

#pragma mark - Chart Specific Properties
#pragma mark Bar Specific
@property (nonatomic) float barWidth;
@property (nonatomic) int barSpacing;
@property (nonatomic) float barGroupSpacing;

#pragma mark Pie Specific
@property (nonatomic) float angle;

#pragma mark Thematic Map Specific
@property (nonatomic) MSGoogleChartMapGeographicArea geographicArea;

#pragma mark - Functions
- (id)initWithChartType:(MSGoogleChartType)type;
- (void)addDataSet:(MSGoogleChartDataSet*)newSet;
- (NSURL*)generateUrl;
- (BOOL)dataComplete;

@end
