//
//  MSGoogleChartUtility.m
//  Galileo-GM
//
//  Created by Drew Breunig on 11/3/11.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

#import "MSGoogleChartUtility.h"
#import "MSGoogleChartDataSet.h"
#import "UIColor-Expanded.h"

@implementation MSGoogleChartUtility

@synthesize baseUrl;

#pragma mark - Global Vars
@synthesize chartType;
@synthesize chartTitle, chartTitleFontSize, chartTitleColor;
@synthesize barSpacing, chartWidth, chartHeight, leftMargin, rightMargin, topMargin, bottomMargin;
@synthesize backGroundColor, chartAreaColor;
@synthesize dataSets;
@synthesize axes;
@synthesize legendPosition;

#pragma mark Bar Specific Properties
@synthesize barWidth, barGroupSpacing;

#pragma mark Pie Specifics
@synthesize angle;

#pragma mark Thematic Map Specific Properties
@synthesize geographicArea;

#pragma mark - Functions


- (NSString*)generateDataString
{
  __block NSMutableString *dataString = [NSMutableString stringWithString:@"chd=t:"];
  __block NSMutableString *titleString = [NSMutableString stringWithString:@"chdl="];
  __block NSMutableString *colorString = [NSMutableString stringWithString:@"chco="];
  __block NSMutableString *lineStyle  = [NSMutableString stringWithString:@"chls="];
  __block BOOL hasTitles  = NO;
  __block BOOL hasColor   = NO;
  __block BOOL isPie = self.chartType == MSGoogleChartTypePie ? YES : NO;
  
  
  [dataSets enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
    
    if ( isPie && idx > 0 ) {
      
    } else {
    
      MSGoogleChartDataSet *data = obj;
      
      if ( idx == 0 && data.title ) {
        hasTitles = YES;
        if ( data.color ) {
          hasColor = YES;
        }
      }
      
      // Add seperators
      if ( idx != 0 && idx < [dataSets count] ) {
        [lineStyle appendString:@"|"];
        [dataString appendString:@"|"];
        if ( data.title && hasTitles ) {
          [titleString appendString:@"|"];
        }
        if ( data.color && hasColor ) {
          [colorString appendString:@","];
        }
      }
      
      // Add data
      [data.values enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        if ( [obj isKindOfClass:[NSNumber class]] ) {
          [dataString appendFormat:@"%1.3f", [obj floatValue]];
        }
        if ( (idx + 1) < [data.values count] ) {
          [dataString appendFormat:@","];
        }
      }];
      
      // Add title
      if ( data.title && hasTitles ) {
        [titleString appendString:[data.title stringByReplacingOccurrencesOfString:@" " withString:@"+"]];
      }
      
      // Add color
      if ( data.color && hasColor ) {
        [colorString appendString:[data.color hexStringFromColor]];
      }
    }
    
  }];
  
  if ( hasTitles ) {
    [dataString appendFormat:@"&%@", titleString];
  }
  if ( hasColor ) {
    [dataString appendFormat:@"&%@", colorString];
  }
  [lineStyle appendString:@"1"];
  
  if ( self.chartType == MSGoogleChartTypeLineChart || self.chartType == MSGoogleChartTypeLineChartSparkLine || self.chartType == MSGoogleChartTypeLineChartXY ) {
    [dataString appendFormat:@"&%@", lineStyle];
  }
    
  return dataString;
}

#pragma mark - Public Methods

- (void)addDataSet:(MSGoogleChartDataSet*)newSet
{
  if ( !dataSets ) {
    self.dataSets = [NSMutableArray arrayWithObject:newSet];
  } else {
    [self.dataSets addObject:newSet];
  }
}

- (NSURL*)generateUrl
{
  if ( ![self dataComplete] ) {
    NSLog(@"Fail");
    return nil;
  }
  
  NSMutableString *urlString = [NSMutableString stringWithString:baseUrl];
  NSLog(@"%@", urlString);
  
  //
  // Required variables
  //
  
  BOOL barChart = NO;
  BOOL sparkLine = NO;
  BOOL pieChart = NO;
  BOOL isMap = NO;
  
  // Chart type
  switch (chartType) {
    case MSGoogleChartTypeBarHorizontalStacked:
      [urlString appendString:@"cht=bhs"];
      barChart = YES;
      break;
    case MSGoogleChartTypeBarVerticalStacked:
      [urlString appendString:@"cht=bvs"];
      barChart = YES;
      break;
    case MSGoogleChartTypeBarVerticalOverlap:
      [urlString appendString:@"cht=bvo"];
      barChart = YES;
      break;
    case MSGoogleChartTypeBarHorizontalGrouped:
      [urlString appendString:@"cht=bhg"];
      barChart = YES;
      break;
    case MSGoogleChartTypeBarVerticalGrouped:
      [urlString appendString:@"cht=bvg"];
      barChart = YES;
      break;
    case MSGoogleChartTypeLineChart:
      [urlString appendString:@"cht=lc"];
      break;
    case MSGoogleChartTypeLineChartSparkLine:
      [urlString appendString:@"cht=ls"];
      sparkLine = YES;
      break;
    case MSGoogleChartTypePie:
      [urlString appendString:@"cht=p"];
      pieChart = YES;
      break;
    case MSGoogleChartTypePie3D:
      [urlString appendString:@"cht=p3"];
      pieChart = YES;
      break;
    case MSGoogleChartTypePie3D:
      [urlString appendString:@"cht=t"];
      isMap = YES;
      break;
    default:
      [urlString appendString:@"cht=bhs"];
      barChart = YES;
      break;
  }
  
  // Auto ranging
  if ( barChart ) {
    [urlString appendFormat:@"&chbh=a,%i", barSpacing];
  } else if ( !pieChart ) {
    [urlString appendFormat:@"&chbh=a"];
  }
  
  // Chart Size
  [urlString appendFormat:@"&chs=%ix%i", chartWidth, chartHeight];
  
  // Data Sets
  if ( [dataSets count] ) {
    [urlString appendFormat:@"&%@", [self generateDataString]];
  }
  
  //
  // Optional Variables
  //
  
  // Chart title
  if ( chartTitle ) {
    [urlString appendFormat:@"&chtt=%@", [chartTitle stringByReplacingOccurrencesOfString:@" " withString:@"+"]];
    // Title color
    if ( chartTitleColor ) {
      [urlString appendFormat:@"&chts=%@", [chartTitleColor hexStringFromColor]];
    }
  }
    
  // Axix Labels
  if ( !sparkLine && !pieChart ) {
    [urlString appendString:@"&chxt=x,y"];
  }
  
  NSLog(@"%@", urlString);
  return [NSURL URLWithString:[urlString stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding]];
}

- (BOOL)dataComplete
{
  if ( !chartWidth || !chartHeight || !chartType || !dataSets ) {
    return NO;
  }
  return YES;
}

- (id)initWithChartType:(MSGoogleChartType)type
{
  self = [super init];
  if ( self ) {
    baseUrl = [NSString stringWithString:@"http://chart.apis.google.com/chart?"];
    chartType = type;
    barSpacing = 5;
  }
  return self;
}

@end
